import hashlib
import json
import os
import time
import uuid
import urllib
import urllib.error

from translation_hub import exceptions
from translation_hub.api import Api
from translation_hub.config import cfg
from translation_hub.core.enums import Languages
from translation_hub.utils.requests import request
from translation_hub.validator_handler import (
    SecretKeyValidatorHandler,
    ContentValidatorHandler,
)
import logging


class YoudaoApi(Api):
    """
    Youdao 翻译 API

    产品说明:
        免费额度: 新人赠送 50 元, 无后续免费额度,每 100 万字符 48 元
        开通地址: https://ai.youdao.com/product-fanyi-text.s
        文档地址: https://ai.youdao.com/DOCSIRMA/html/trans/api/wbfy/index.html

    使用说明:
        Youdao 翻译没有每月的免费额度，需要充值后才能使用，中英文翻译的准确度较高，有冗余部分，默认会返回翻译结果的发音链接，不能很好的控制。

    正确返回值示例:
        {
          "tSpeakUrl":"https://openapi.youdao.com/ttsapi?q=Hello%2C+world&langType=en-USA&sign=BA31E66617027E90255441CED4DAD187&salt=1733782913720&voice=4&format=mp3&appKey=2e276409e88add8a&ttsVoiceStrict=false&osType=api",
          "requestId":"025cab58-9bba-4726-9ec5-ee6dccfde971",
          "query":"你好,世界",
          "isDomainSupport":false,
          "translation":[
            "Hello, world"
          ],
          "mTerminalDict":{
            "url":"https://m.youdao.com/m/result?lang=zh-CHS&word=%E4%BD%A0%E5%A5%BD%2C%E4%B8%96%E7%95%8C"
          },
          "errorCode":"0",
          "dict":{
            "url":"yddict://m.youdao.com/dict?le=eng&q=%E4%BD%A0%E5%A5%BD%2C%E4%B8%96%E7%95%8C"
          },
          "webdict":{
            "url":"http://mobile.youdao.com/dict?le=eng&q=%E4%BD%A0%E5%A5%BD%2C%E4%B8%96%E7%95%8C"
          },
          "l":"zh-CHS2en",
          "isWord":false,
          "speakUrl":"https://openapi.youdao.com/ttsapi?q=%E4%BD%A0%E5%A5%BD%2C%E4%B8%96%E7%95%8C&langType=zh-CHS&sign=BDDE59DC15C7EF09FADD69BAD437F538&salt=1733782913720&voice=4&format=mp3&appKey=2e276409e88add8a&ttsVoiceStrict=false&osType=api"
        }

    错误返回值示例:
        {"requestId":"1e8dfafb-2605-4061-b807-852dd750892f","errorCode":"108","l":"源语言2目标语言"}
    """

    api_url = "https://openapi.youdao.com/api"
    error_codes = {
        101: "缺少必填的参数,首先确保必填参数齐全，然后确认参数书写是否正确。",
        102: "不支持的语言类型",
        103: "翻译文本过长",
        104: "不支持的API类型",
        105: "不支持的签名类型",
        106: "不支持的响应类型",
        107: "不支持的传输加密类型",
        108: "应用ID无效，注册账号，登录后台创建应用并完成绑定，可获得应用ID和应用密钥等信息",
        109: "batchLog格式不正确",
        110: "无相关服务的有效应用,应用没有绑定服务应用，可以新建服务应用。注：某些服务的翻译结果发音需要tts服务，需要在控制台创建语音合成服务绑定应用后方能使用。",
        111: "开发者账号无效",
        112: "请求服务无效",
        113: "q不能为空",
        114: "不支持的图片传输方式",
        116: "strict字段取值无效，请参考文档填写正确参数值",
        201: "解密失败，可能为DES,BASE64,URLDecode的错误",
        202: "签名检验失败,如果确认应用ID和应用密钥的正确性，仍返回202，一般是编码问题。请确保翻译文本 q 为UTF-8编码.",
        203: "访问IP地址不在可访问IP列表",
        205: "请求的接口与应用的平台类型不一致，确保接入方式（Android SDK、IOS SDK、API）与创建的应用平台类型一致。如有疑问请参考入门指南",
        206: "因为时间戳无效导致签名校验失败",
        207: "重放请求",
        301: "辞典查询失败",
        302: "翻译查询失败",
        303: "服务端的其它异常",
        304: "翻译失败，请联系技术同学",
        308: "rejectFallback参数错误",
        309: "domain参数错误",
        310: "未开通领域翻译服务",
        401: "账户已经欠费，请进行账户充值",
        402: "offlinesdk不可用",
        411: "访问频率受限,请稍后访问",
        412: "长请求过于频繁，请稍后访问",
        1001: "无效的OCR类型",
        1002: "不支持的OCR image类型",
        1003: "不支持的OCR Language类型",
        1004: "识别图片过大",
        1201: "图片base64解密失败",
        1301: "OCR段落识别失败",
        1411: "访问频率受限",
        1412: "超过最大识别字节数",
        2003: "不支持的语言识别Language类型",
        2004: "合成字符过长",
        2005: "不支持的音频文件类型",
        2006: "不支持的发音类型",
        2201: "解密失败",
        2301: "服务的异常",
        2411: "访问频率受限,请稍后访问",
        2412: "超过最大请求字符数",
        3001: "不支持的语音格式",
        3002: "不支持的语音采样率",
        3003: "不支持的语音声道",
        3004: "不支持的语音上传类型",
        3005: "不支持的语言类型",
        3006: "不支持的识别类型",
        3007: "识别音频文件过大",
        3008: "识别音频时长过长",
        3009: "不支持的音频文件类型",
        3010: "不支持的发音类型",
        3201: "解密失败",
        3301: "语音识别失败",
        3302: "语音翻译失败",
        3303: "服务的异常",
        3411: "访问频率受限,请稍后访问",
        3412: "超过最大请求字符数",
        4001: "不支持的语音识别格式",
        4002: "不支持的语音识别采样率",
        4003: "不支持的语音识别声道",
        4004: "不支持的语音上传类型",
        4005: "不支持的语言类型",
        4006: "识别音频文件过大",
        4007: "识别音频时长过长",
        4201: "解密失败",
        4301: "语音识别失败",
        4303: "服务的异常",
        4411: "访问频率受限,请稍后访问",
        4412: "超过最大请求时长",
        5001: "无效的OCR类型",
        5002: "不支持的OCR image类型",
        5003: "不支持的语言类型",
        5004: "识别图片过大",
        5005: "不支持的图片类型",
        5006: "文件为空",
        5201: "解密错误，图片base64解密失败",
        5301: "OCR段落识别失败",
        5411: "访问频率受限",
        5412: "超过最大识别流量",
        9001: "不支持的语音格式",
        9002: "不支持的语音采样率",
        9003: "不支持的语音声道",
        9004: "不支持的语音上传类型",
        9005: "不支持的语音识别 Language类型",
        9301: "ASR识别失败",
        9303: "服务器内部错误",
        9411: "访问频率受限（超过最大调用次数）",
        9412: "超过最大处理语音长度",
        10001: "无效的OCR类型",
        10002: "不支持的OCR image类型",
        10004: "识别图片过大",
        10201: "图片base64解密失败",
        10301: "OCR段落识别失败",
        10411: "访问频率受限",
        10412: "超过最大识别流量",
        11001: "不支持的语音识别格式",
        11002: "不支持的语音识别采样率",
        11003: "不支持的语音识别声道",
        11004: "不支持的语音上传类型",
        11005: "不支持的语言类型",
        11006: "识别音频文件过大",
        11007: "识别音频时长过长，最大支持30s",
        11201: "解密失败",
        11301: "语音识别失败",
        11303: "服务的异常",
        11411: "访问频率受限,请稍后访问",
        11412: "超过最大请求时长",
        12001: "图片尺寸过大",
        12002: "图片base64解密失败",
        12003: "引擎服务器返回错误",
        12004: "图片为空",
        12005: "不支持的识别图片类型",
        12006: "图片无匹配结果",
        13001: "不支持的角度类型",
        13002: "不支持的文件类型",
        13003: "表格识别图片过大",
        13004: "文件为空",
        13301: "表格识别失败",
        15001: "需要图片",
        15002: "图片过大（1M）",
        15003: "服务调用失败",
        17001: "需要图片",
        17002: "图片过大（1M）",
        17003: "识别类型未找到",
        17004: "不支持的识别类型",
        17005: "服务调用失败",
    }

    def __init__(self, api_id: str | None = None, api_key: str | None = None):
        self.api_id, self.api_key = self._get_api_id_and_key(api_id, api_key)

        self.content_validator_handler = ContentValidatorHandler()
        secret_key_validator_handler = SecretKeyValidatorHandler()
        # 验证 API Key
        is_api_key_valid = secret_key_validator_handler.validate(self.api_key)

        if not is_api_key_valid:
            raise exceptions.InvalidSecretKeyError(
                f"Invalid API Key, please check and try again."
                f"current api_key: {self.api_key}"
            )

    def translate(
        self,
        text: str,
        source: Languages | str = Languages.English,
        target: Languages | str = Languages.Chinese,
    ) -> str:
        text = text.strip()

        is_content_valid: bool = self.content_validator_handler.validate(text)
        if not is_content_valid:
            raise exceptions.InvalidContentError(f"Invalid content: {text}")

        source = self._trans_language(source)
        target = self._trans_language(target)

        def encrypt(sign_str):
            hash_algorithm = hashlib.sha256()
            hash_algorithm.update(sign_str.encode("utf-8"))
            return hash_algorithm.hexdigest()

        def truncate(q):
            if q is None:
                return None
            size = len(q)
            return q if size <= 20 else q[0:10] + str(size) + q[size - 10 : size]

        def do_request(data):
            headers = {"Content-Type": "application/x-www-form-urlencoded"}
            return request(self.api_url, data=data, headers=headers)

        data = {"from": source, "to": target, "signType": "v3"}
        current_time = str(int(time.time()))
        data["curtime"] = current_time
        salt = str(uuid.uuid1())
        sign_str = self.api_id + truncate(text) + salt + current_time + self.api_key
        sign = encrypt(sign_str)
        data["appKey"] = self.api_id
        data["q"] = text
        data["salt"] = salt
        data["sign"] = sign

        try:
            response = do_request(data)
            logging.debug(f"请求成功: {response}")
        except urllib.error.HTTPError as e:
            raise exceptions.RequestError(f"网络请求错误: {e}")
        except Exception as e:
            raise exceptions.UnknownError(f"未知错误: {e}")

        try:
            response_data = json.loads(response)
            if response_data["errorCode"] != "0":
                error_code = response_data["errorCode"]
                error_msg = self.error_codes.get(int(error_code), "未知错误")
                raise exceptions.RequestError(f"请求错误: {error_msg}")
            result = response_data["translation"][0]

        except json.JSONDecodeError as e:
            raise exceptions.JsonDecodeError(f"Failed to decode response: {e}")
        except KeyError as e:
            raise exceptions.JsonDecodeError(f"Failed to decode response: {e}")

        return result

    def _trans_language(self, languages: Languages | str) -> str:
        """将语言转换为 Deepl 支持的语言代码"""
        if isinstance(languages, Languages):
            languages = languages.value

        trans_dict: dict[str, str] = {
            "auto": "auto",
            "zh": "zh-CHS",
            "en": "en",
            "ru": "ru",
            "ja": "ja",
            "kor": "ko",
        }

        return trans_dict.get(languages) or languages

    def _get_api_id_and_key(
        self, api_id: str | None, api_key: str | None
    ) -> tuple[str, str]:
        """先从环境变量中获取，如果没有则从配置文件中获取"""
        if api_id and api_key:
            return api_id, api_key

        api_id = os.environ.get("YoudaoAppId")
        api_key = os.environ.get("YoudaoSecretKey")
        if api_id and api_key:
            return api_id, api_key

        api_id = cfg.get(cfg.YoudaoAppId)
        api_key = cfg.get(cfg.YoudaoSecretKey)
        if api_id and api_key:
            return api_id, api_key

        raise exceptions.InvalidSecretKeyError(
            f"Invalid API ID or API Key, please check and try again."
            f"current api_id: {api_id}, current api_key: {api_key}"
        )



from translation_hub.apis.google_api import GoogleApi
from translation_hub.core.enums import Languages
import pytest


class TestGoogleApi:
    @pytest.fixture
    def google_api(self):
        return GoogleApi()

    def test_translate_successful(self, google_api: GoogleApi):
        """测试正常翻译流程"""
        # 中文翻译为日文
        result = google_api.translate(
            "你吃饭了么?", Languages.Chinese, Languages.Japanese
        )
        assert result == "食べましたか？"

        # 英文翻译为中文
        result = google_api.translate(
            "about your situation", Languages.English, Languages.Chinese
        )
        assert result == "关于你的情况"

        # 英语翻译成韩文
        result = google_api.translate(
            "about your situation", Languages.English, Languages.Korea
        )
        assert result == "당신의 상황에 대해서"

        # 英文翻译成俄文
        result = google_api.translate(
            "about your situation", Languages.English, Languages.Russia
        )
        assert result == "о вашей ситуации"

        # 自动翻译成中文
        result = google_api.translate(
            "about your situation", Languages.Auto, Languages.Chinese
        )
        assert result == "关于你的情况"



import hashlib
import hmac
import json
import os
import base64
import time
import urllib.parse

from translation_hub import exceptions
from translation_hub.api import Api
from translation_hub.config import cfg
from translation_hub.core.enums import Languages
from translation_hub.utils.requests import request
from translation_hub.validator_handler import (
    SecretKeyValidatorHandler,
    ContentValidatorHandler,
)
import logging
from typing import Literal


class AliyunApi(Api):
    """
    阿里云翻译 API

    产品说明:
        免费额度: 100万 字/月
        开通地址: https://www.aliyun.com/product/ai/alimt
        文档地址: https://help.aliyun.com/zh/machine-translation/

    使用说明:
        阿里云的app_id和app_key注册也比较麻烦，请参考: https://help.aliyun.com/zh/ram/user-guide/create-an-accesskey-pair

    正确返回值示例:
        {
            "RequestId": "F3D8EEF0-1474-53B9-A3BB-E10D6F696D3D",
            "Data": {
                "WordCount": "13",
                "Translated": "你好，世界!"
            },
            "Code": "200"
        }


    错误返回值示例:
        {
            "Message": "语种拼写错误",
            "RequestId": "5605A851-C90E-57B5-A002-47AFEF3F961F",
            "Code": "10033"
        }
    """

    api_url = "http://mt.cn-hangzhou.aliyuncs.com/"

    def __init__(
        self,
        api_id: str | None = None,
        api_key: str | None = None,
        version: Literal["normal", "pro"] = "normal",
    ):
        """
        初始化阿里云翻译 API
        Args:
            api_id: 阿里云API ID
            api_key: 阿里云API Key
            version: 翻译版本，normal: 通用翻译，pro: 专业版翻译(需要额外开通)
        """
        self.version = version
        self.api_id, self.api_key = self._get_api_id_and_key(api_id, api_key)

        self.content_validator_handler = ContentValidatorHandler()
        secret_key_validator_handler = SecretKeyValidatorHandler()

        is_api_id_valid = secret_key_validator_handler.validate(self.api_id)
        is_api_key_valid = secret_key_validator_handler.validate(self.api_key)

        if not is_api_id_valid or not is_api_key_valid:
            raise exceptions.InvalidSecretKeyError(
                f"Invalid API ID or API Key, please check and try again."
                f"current api_id: {self.api_id}, current api_key: {self.api_key}"
            )

    def translate(
        self,
        text: str,
        source: Languages | str = Languages.English,
        target: Languages | str = Languages.Chinese,
    ) -> str:
        """执行翻译请求"""
        text = text.strip()

        is_content_valid: bool = self.content_validator_handler.validate(text)
        if not is_content_valid:
            raise exceptions.InvalidContentError(f"Invalid content: {text}")

        source = self._trans_language(source)
        target = self._trans_language(target)

        # 构造参数
        parameters = {
            "Action": "TranslateGeneral",
            "FormatType": "text",
            "SourceLanguage": source,
            "TargetLanguage": target,
            "SourceText": text,
            "Scene": "general" if self.version == "normal" else "ecommerce",
            "Format": "JSON",
            "Version": "2018-10-12",
            "AccessKeyId": self.api_id,
            "SignatureMethod": "HMAC-SHA1",
            "Timestamp": time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime()),
            "SignatureVersion": "1.0",
            "SignatureNonce": str(int(time.time() * 1000)),
            "RegionId": "cn-hangzhou",
        }

        # 计算签名
        parameters["Signature"] = self._calculate_signature(parameters)

        try:
            response = request(url=self.api_url, headers={}, payload=parameters)
            logging.debug(f"请求成功: {response}")
            response_data = json.loads(response)

            if "Code" in response_data and response_data["Code"] != "200":
                error_code = response_data.get("Code")
                error_msg = response_data.get("Message", "Unknown error")
                raise exceptions.ServerError(f"服务器错误: {error_code} - {error_msg}")

        except json.JSONDecodeError as e:
            raise exceptions.JsonDecodeError(f"解析响应失败: {e}")

        try:
            result = response_data["Data"]["Translated"]
            return result
        except KeyError:
            raise exceptions.ResponseError(f"获取翻译结果失败: {response_data}")

    def _trans_language(self, language: Languages | str) -> str:
        """转换语言代码为阿里云支持的格式"""
        if isinstance(language, Languages):
            language = language.value

        trans_dict = {
            "auto": "auto",
            "zh": "zh",
            "en": "en",
            "ja": "ja",
            "kor": "ko",
            "ru": "ru",
        }
        return trans_dict.get(language) or language

    def _get_api_id_and_key(
        self, api_id: str | None, api_key: str | None
    ) -> tuple[str, str]:
        """获取 API ID 和 Key"""
        if api_id and api_key:
            return api_id, api_key

        api_id = os.environ.get("AliyunAppId")
        api_key = os.environ.get("AliyunSecretKey")
        if api_id and api_key:
            return api_id, api_key

        api_id = cfg.get(cfg.AliyunAppId)
        api_key = cfg.get(cfg.AliyunSecretKey)
        if api_id and api_key:
            return api_id, api_key

        raise exceptions.InvalidSecretKeyError(
            f"Invalid API ID or API Key, please check and try again."
            f"current api_id: {api_id}, current api_key: {api_key}"
        )

    def _calculate_signature(self, parameters: dict) -> str:
        """计算阿里云API签名"""
        sorted_params = sorted(parameters.items())
        canonicalized_query = "&".join(
            [f"{self._encode(k)}={self._encode(v)}" for k, v in sorted_params]
        )

        string_to_sign = "GET&%2F&" + self._encode(canonicalized_query)

        h = hmac.new(
            f"{self.api_key}&".encode("utf-8"),
            string_to_sign.encode("utf-8"),
            hashlib.sha1,
        )
        return base64.b64encode(h.digest()).decode()

    def _encode(self, value: str) -> str:
        """URL编码"""
        return (
            urllib.parse.quote(str(value), safe="~")
            .replace("+", "%20")
            .replace("*", "%2A")
            .replace("%7E", "~")
        )



from translation_hub.utils.config import QConfig, ConfigItem, qconfig
from translation_hub.core.paths import CONFIG_FILE
import logging


class Config(QConfig):
    BaiduAppId = ConfigItem("Baidu", "BaiduAppId", "")
    BaiduSecretKey = ConfigItem("Baidu", "BaiduSecretKey", "")
    DeeplApiKey = ConfigItem("Deepl", "DeeplApiKey", "")
    YoudaoAppId = ConfigItem("Youdao", "YoudaoAppId", "")
    YoudaoSecretKey = ConfigItem("Youdao", "YoudaoSecretKey", "")
    TencentAppId = ConfigItem("Tencent", "TencentAppId", "")
    TencentSecretKey = ConfigItem("Tencent", "TencentSecretKey", "")
    AliyunAppId = ConfigItem("Aliyun", "AliyunAppId", "")
    AliyunSecretKey = ConfigItem("Aliyun", "AliyunSecretKey", "")


cfg = Config()
cfg.file = CONFIG_FILE
if not CONFIG_FILE.exists():
    cfg.save()
    logging.info(f"Config file not found, create a new one at {CONFIG_FILE}")
qconfig.load(CONFIG_FILE, cfg)



import hashlib
import time
import urllib.request
import urllib.parse
import json
import uuid
from typing import Optional, Dict


def request(
    url: str,
    headers: Dict[str, str],
    payload: Optional[Dict[str, str]] = None,
    data: Optional[Dict[str, str]] = None,
    json_data: Optional[Dict] = None,
) -> str:
    """
    发送 HTTP 请求并返回响应内容。

    参数:
        url (str): 请求的 URL。
        headers (dict): 请求头。
        payload (dict, optional): 查询参数，将附加到 URL 中。
        data (dict, optional): 表单数据（application/x-www-form-urlencoded）。
        json_data (dict, optional): JSON 数据（application/json）。

    返回:
        str: 响应的内容。

    示例:
        response = request(
            url="https://api.example.com/data",
            headers={"Authorization": "Bearer token"},
            payload={"query": "value"},
            data={"key": "value"},
            json_data={"json_key": "json_value"}
        )
    """
    # 处理查询参数（payload）
    if payload:
        query_string = urllib.parse.urlencode(payload)
        separator = "&" if "?" in url else "?"
        url = f"{url}{separator}{query_string}"

    # 初始化请求数据和Content-Type
    request_data = None
    if json_data is not None:
        request_data = json.dumps(json_data).encode("utf-8")
        headers["Content-Type"] = "application/json"
    elif data is not None:
        request_data = urllib.parse.urlencode(data).encode("utf-8")
        headers["Content-Type"] = "application/x-www-form-urlencoded"

    # 创建请求对象
    req = urllib.request.Request(url, headers=headers, data=request_data)

    with urllib.request.urlopen(req) as response:
        charset = response.headers.get_content_charset() or "utf-8"
        return response.read().decode(charset)


# if __name__ == "__main__":
#     # 示例 1: GET 请求，带查询参数
#     response = request(
#         url="https://httpbin.org/get",
#         headers={"User-Agent": "Mozilla/5.0"},
#         payload={"search": "python", "page": "1"},
#     )
#     print("GET Response:", response)
#
#     # 示例 2: POST 请求，发送表单数据
#     response = request(
#         url="https://httpbin.org/post",
#         headers={"User-Agent": "Mozilla/5.0"},
#         data={"username": "test", "password": "1234"},
#     )
#     print("POST Form Response:", response)
#
#     # 示例 3: POST 请求，发送 JSON 数据
#     response = request(
#         url="https://httpbin.org/post",
#         headers={"User-Agent": "Mozilla/5.0"},
#         json_data={"key": "value", "number": 42},
#     )
#     print("POST JSON Response:", response)

if __name__ == "__main__":
    YOUDAO_URL = "https://openapi.youdao.com/api"
    APP_KEY = "2e276409e88add8a"
    APP_SECRET = "JXROFWyhCl8REgF9f8M6k4LAc2FK4HUv"

    def encrypt(sign_str):
        hash_algorithm = hashlib.sha256()
        hash_algorithm.update(sign_str.encode("utf-8"))
        return hash_algorithm.hexdigest()

    def truncate(q):
        if q is None:
            return None
        size = len(q)
        return q if size <= 20 else q[0:10] + str(size) + q[size - 10 : size]

    def do_request(data):
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        return request(YOUDAO_URL, data=data, headers=headers)

    q = "你好,世界"

    data = {}
    data["from"] = "源语言"
    data["to"] = "目标语言"
    data["signType"] = "v3"
    curtime = str(int(time.time()))
    data["curtime"] = curtime
    salt = str(uuid.uuid1())
    signStr = APP_KEY + truncate(q) + salt + curtime + APP_SECRET
    sign = encrypt(signStr)
    data["appKey"] = APP_KEY
    data["q"] = q
    data["salt"] = salt
    data["sign"] = sign

    response = do_request(data)
    print(response)



import json
import os
import time
import urllib
import urllib.error

import pytest
from translation_hub.apis.tencent_api import TencentApi
from translation_hub.core.enums import Languages
from unittest.mock import patch, MagicMock
from translation_hub import exceptions
from translation_hub.config import cfg


@pytest.fixture
def tencent_api():
    return TencentApi(
        api_id=cfg.get(cfg.TencentAppId),
        api_key=cfg.get(cfg.TencentSecretKey),
    )


class TestTencentApi:
    @patch.dict(
        os.environ, {"TencentAppId": "env_api_id", "TencentSecretKey": "env_api_key"}
    )
    def test_get_api_id_and_key_from_env(self):
        api = TencentApi()
        assert api.api_id == "env_api_id"
        assert api.api_key == "env_api_key"

    @patch.object(
        cfg,
        "get",
        side_effect=lambda key: "cfg_api_id"
        if key == cfg.TencentAppId
        else "cfg_api_key",
    )
    def test_get_api_id_and_key_from_cfg(self, mock_cfg):
        api = TencentApi()
        assert api.api_id == "cfg_api_id"
        assert api.api_key == "cfg_api_key"

    def test_get_api_id_and_key_directly(self):
        api = TencentApi(api_id="direct_api_id", api_key="direct_api_key")
        assert api.api_id == "direct_api_id"
        assert api.api_key == "direct_api_key"

    @patch.dict(os.environ, {}, clear=True)
    @patch.object(cfg, "get", return_value=None)
    def test_get_api_id_and_key_error(self, mock_cfg):
        with pytest.raises(exceptions.InvalidSecretKeyError):
            TencentApi()

    def test_translates_text_correctly(self, tencent_api):
        result = tencent_api.translate(
            "Hello, world!", Languages.English, Languages.Chinese
        )
        assert result == "你好，世界！"

    def test_handles_invalid_content_error(self, tencent_api):
        with pytest.raises(exceptions.InvalidContentError):
            tencent_api.translate("", Languages.English, Languages.Chinese)

    def test_request_error(self, tencent_api):
        with patch(
            "translation_hub.apis.tencent_api.urllib.request.urlopen"
        ) as mock_urlopen:
            # 创建一个模拟的响应对象
            # 配置mock_urlopen的返回值，以支持with语句
            with pytest.raises(exceptions.RequestError):
                mock_urlopen.side_effect = urllib.error.HTTPError(
                    url="", code=403, msg="", hdrs=None, fp=None
                )
                tencent_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_server_error(self, tencent_api):
        with patch(
            "translation_hub.apis.tencent_api.urllib.request.urlopen"
        ) as mock_urlopen:
            # 创建一个模拟的上下文管理器对象
            mock_cm = MagicMock()
            mock_response = mock_cm.__enter__.return_value
            mock_response.read.return_value = json.dumps(
                {
                    "Response": {
                        "Error": {
                            "Code": "FailedOperation",
                            "Message": "Server error occurred",
                        },
                        "RequestId": "test-request-id",
                    }
                }
            ).encode("utf-8")
            mock_urlopen.return_value = mock_cm

            with pytest.raises(exceptions.ServerError):
                tencent_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_http_error(self, tencent_api):
        with patch(
            "urllib.request.urlopen",
            side_effect=urllib.error.HTTPError(
                url="", code=403, msg="", hdrs=None, fp=None
            ),
        ):
            with pytest.raises(exceptions.RequestError):
                tencent_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_json_decode_error(self, tencent_api):
        with patch(
            "translation_hub.apis.tencent_api.urllib.request.urlopen"
        ) as mock_urlopen:
            # 创建一个模拟的上下文管理器对象
            mock_cm = MagicMock()
            mock_response = mock_cm.__enter__.return_value
            mock_response.read.return_value = b"Invalid JSON {"
            mock_urlopen.return_value = mock_cm

            with pytest.raises(exceptions.JsonDecodeError):
                tencent_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_generic_exception(self, tencent_api):
        with patch("urllib.request.urlopen", side_effect=Exception("Some error")):
            with pytest.raises(exceptions.UnknownError):
                tencent_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_trans_language(self, tencent_api):
        assert tencent_api._trans_language(Languages.English) == "en"
        assert tencent_api._trans_language("en") == "en"

    def test_get_authorization(self, tencent_api):
        payload = {
            "SourceText": "Hello, world!",
            "Source": "en",
            "Target": "zh",
            "ProjectId": 0,
        }
        timestamp = int(time.time())
        authorization = tencent_api._get_authorization(payload, timestamp)
        assert "TC3-HMAC-SHA256" in authorization



import json

import pytest
import os
from unittest.mock import patch
from translation_hub.apis.aliyun_api import AliyunApi
from translation_hub.core.enums import Languages
from translation_hub import exceptions
from translation_hub.config import cfg


class TestAliyunApi:
    @pytest.fixture
    def api_credentials(self):
        return {
            "api_id": cfg.get(cfg.AliyunAppId),
            "api_key": cfg.get(cfg.AliyunSecretKey),
        }

    @pytest.fixture
    def aliyun_api(self, api_credentials):
        return AliyunApi(
            api_id=api_credentials["api_id"], api_key=api_credentials["api_key"]
        )

    @pytest.fixture
    def mock_successful_response(self):
        return {
            "RequestId": "test-request-id",
            "Data": {"WordCount": "2", "Translated": "你好"},
            "Code": "200",
        }

    def test_init_with_valid_credentials(self, api_credentials):
        """测试使用有效凭据初始化API"""
        api = AliyunApi(
            api_id=api_credentials["api_id"], api_key=api_credentials["api_key"]
        )
        assert api.api_id == api_credentials["api_id"]
        assert api.api_key == api_credentials["api_key"]

    def test_init_with_environment_variables(self):
        """测试从环境变量获取凭据"""
        with patch.dict(
            os.environ,
            {"AliyunAppId": "env_idaaaaaaaaaa", "AliyunSecretKey": "env_keyaaaaaaaaaa"},
        ):
            api = AliyunApi()
            assert api.api_id == "env_idaaaaaaaaaa"
            assert api.api_key == "env_keyaaaaaaaaaa"

    def test_translate_successful(self, aliyun_api, mock_successful_response):
        """测试正常翻译流程"""
        result = aliyun_api.translate("Hello", Languages.English, Languages.Chinese)
        assert result == "你好"

    def test_translate_server_error(self, aliyun_api):
        """测试服务器错误情况"""
        error_response = {
            "Code": "10033",
            "Message": "语种拼写错误",
            "RequestId": "test-error-id",
        }
        with patch("translation_hub.apis.aliyun_api.request") as mock_request:
            mock_request.return_value = json.dumps(error_response)
            with pytest.raises(exceptions.ServerError):
                aliyun_api.translate("Hello", "random", Languages.Chinese)

    def test_trans_language_with_enum(self, aliyun_api):
        """测试语言代码转换 - 枚举输入"""
        assert aliyun_api._trans_language(Languages.Chinese) == "zh"
        assert aliyun_api._trans_language(Languages.English) == "en"
        assert aliyun_api._trans_language(Languages.Japanese) == "ja"



from translation_hub import exceptions
from translation_hub.api import Api
import re
import html
from urllib import parse
from urllib import request
import urllib.error
from translation_hub.core.enums import Languages
from translation_hub.validator_handler import (
    ContentValidatorHandler,
)


class GoogleApi(Api):
    """
    谷歌翻译 API(需要翻墙,无需秘钥,免费)

    谷歌翻译随时可能会改变接口，不保证长期可用。目前测试正常。
    使用的时候注意请求频率，不要频繁请求，否则可能会被谷歌限制。
    """

    api_url = "http://translate.google.com/m?q=%s&tl=%s&sl=%s"

    def __init__(self):
        self.content_validator_handler = ContentValidatorHandler()

    def translate(
        self,
        text: str,
        source: Languages | str = Languages.English,
        target: Languages | str = Languages.Chinese,
    ):
        text: str = text.strip()

        is_content_valid: bool = self.content_validator_handler.validate(text)
        if not is_content_valid:
            raise exceptions.InvalidContentError(f"Invalid content: {text}")

        source: str = self._trans_language(source)
        target: str = self._trans_language(target)

        # 该方法来自 https://blog.csdn.net/wtl1992/article/details/135117192
        text = parse.quote(text)
        url = self.api_url % (text, target, source)

        # 使用 urllib 发起请求
        try:
            response = request.urlopen(url)
            data = response.read().decode("utf-8")
        except urllib.error.HTTPError as e:
            raise exceptions.RequestError(str(e))
        except urllib.error.URLError as e:
            raise exceptions.RequestError(str(e))
        except Exception as e:
            raise exceptions.UnknownError(str(e))

        expr = r'(?s)class="(?:t0|result-container)">(.*?)<'
        result = re.findall(expr, data)
        if len(result) == 0:
            return ""

        return html.unescape(result[0])

    def _trans_language(self, language: Languages | str) -> str:
        """转换语言代码为支持的格式"""
        if isinstance(language, Languages):
            language = language.value

        trans_dict = {
            "auto": "auto",
            "zh": "zh-CN",
            "en": "en",
            "ja": "ja",
            "kor": "ko",
            "ru": "ru",
        }
        return trans_dict.get(language) or language



from enum import Enum


class Languages(Enum):
    Auto = "auto"  # 自动检测(需要目标API支持, translatehub不提供检测)
    Chinese = "zh"  # 中文
    English = "en"  # 英文
    Russia = "ru"  # 俄文
    Japanese = "ja"  # 日文
    Korea = "kor"  # 韩文



class RequestError(Exception):
    def __init__(self, message="请求错误: 请重试"):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"RequestError: {self.message}"


class ResponseError(Exception):
    def __init__(self, message="响应错误: 请重试"):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"ResponseError: {self.message}"


class ServerError(Exception):
    def __init__(self, message="服务器错误: 请重试"):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"ServerError: {self.message}"


class InvalidSecretKeyError(Exception):
    def __init__(self, message="密钥错误: 请检查密钥是否正确"):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"InvalidSecretKeyError: {self.message}"


class InvalidContentError(Exception):
    def __init__(self, message="请求内容错误: 请检查请求内容"):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"InvalidContentError: {self.message}"


class JsonDecodeError(Exception):
    def __init__(self, message="无法解码 JSON: 不合法的 JSON"):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"JsonDecodeError: {self.message}"


class UnknownError(Exception):
    def __init__(self, message="未知错误: 请重试或者前往github提交issue"):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"UnknownError: {self.message}"



from translation_hub.validators.validator import Validator


class EmptyStringValidator(Validator):
    def validate(self, value: str) -> bool:
        return bool(value)


class FullNumberValidator(Validator):
    def validate(self, value: str) -> bool:
        return value.isnumeric()



import json
import os

from translation_hub import exceptions
from translation_hub.api import Api
from translation_hub.config import cfg
from translation_hub.core.enums import Languages
from translation_hub.utils.requests import request
from translation_hub.validator_handler import (
    SecretKeyValidatorHandler,
    ContentValidatorHandler,
)
import urllib
import urllib.error
import logging


class DeeplApi(Api):
    """
    Deepl API

    产品说明:
        免费额度: 50万 字/月
        开通地址: https://www.deepl.com/zh/pro-api
        文档地址: https://developers.deepl.com/docs/api-reference

    使用说明:
        deepl 对于中英文翻译的准确度较高，需要注册账号并创建应用获取 API Key。

    正确返回值示例:
        {"translations":[{"detected_source_language":"EN","text":"你好，世界"}]}

    错误返回值示例:
        HTTP 429: too many requests
        HTTP 456: quota exceeded
        HTTP 500: internal server error
    """

    api_url = "https://api-free.deepl.com/v2/translate"

    def __init__(self, api_key: str | None = None):
        self.api_key = self._get_api_id_and_key(api_key)

        self.content_validator_handler = ContentValidatorHandler()
        secret_key_validator_handler = SecretKeyValidatorHandler()
        # 验证 API Key
        is_api_key_valid = secret_key_validator_handler.validate(self.api_key)
        logging.debug(self.api_key)

        if not is_api_key_valid:
            raise exceptions.InvalidSecretKeyError(
                f"Invalid API Key, please check and try again."
                f"current api_key: {self.api_key}"
            )

    def translate(
        self,
        text: str,
        source: Languages | str = Languages.English,
        target: Languages | str = Languages.Chinese,
    ) -> str:
        text = text.strip()

        is_content_valid: bool = self.content_validator_handler.validate(text)
        if not is_content_valid:
            raise exceptions.InvalidContentError(f"Invalid content: {text}")

        source = self._trans_language(source)
        target = self._trans_language(target)

        # deepl 的 API key 是放在 header 的 Authorization 头中的
        headers = {
            "Authorization": f"DeepL-Auth-Key {self.api_key}",
            "Content-Type": "application/json",
        }

        # deepl没有source_lang参数就能自动识别
        if source == "auto":
            json_data = {"text": [text], "target_lang": target}
        else:
            json_data = {"text": [text], "source_lang": source, "target_lang": target}

        try:
            response = request(self.api_url, headers, json_data=json_data)
            logging.debug(f"请求成功: {response}")

        except urllib.error.HTTPError as e:
            match e.code:
                case 403:
                    raise exceptions.RequestError("请求被拒绝: 请检查密钥是否正确")
                case 429:
                    raise exceptions.RequestError("请求次数过多: 请稍后再试")
                case 456:
                    raise exceptions.RequestError("超出配额: 请稍后再试")
                case 500:
                    raise exceptions.ServerError("服务器错误: 请稍后再试")
            raise exceptions.UnknownError(str(e))
        except Exception as e:
            raise exceptions.UnknownError(str(e))

        try:
            response_data = json.loads(response)
            result = response_data["translations"][0]["text"]
        except json.JSONDecodeError as e:
            raise exceptions.JsonDecodeError(f"无法解码 JSON: 不合法的 JSON: {e}")
        except KeyError as e:
            raise exceptions.ResponseError(f"响应错误: {response},错误: {e}")

        return result

    def _trans_language(self, languages: Languages | str) -> str:
        """将语言转换为 Deepl 支持的语言代码"""
        if isinstance(languages, Languages):
            languages = languages.value

        trans_dict: dict[str, str] = {
            "auto": "auto",
            "zh": "ZH",
            "en": "EN",
            "ru": "RU",
            "ja": "JA",
            "kor": "KO",
        }

        return trans_dict.get(languages) or languages

    def _get_api_id_and_key(self, api_key: str | None) -> str:
        """先从环境变量中获取，如果没有则从配置文件中获取"""
        if api_key:
            return api_key

        api_key = os.environ.get("DeeplApiKey")
        if api_key:
            return api_key

        api_key = cfg.get(cfg.DeeplApiKey)
        if api_key:
            return api_key

        raise exceptions.InvalidSecretKeyError(
            f"Invalid API Key, please check and try again."
            f"current api_key: {api_key}"
        )



from translation_hub.validators.validator import Validator
from translation_hub.validators import secret_key_validator, content_validator
from abc import ABC


class ValidatorHandler(ABC):
    def __init__(self):
        self.validators = []

    def add_validator(self, validator: Validator):
        self.validators.append(validator)

    def validate(self, data: str) -> bool:
        for validator in self.validators:
            if not validator.validate(data):
                return False
        return True


class SecretKeyValidatorHandler(ValidatorHandler):
    def __init__(self):
        super().__init__()
        self.validators.extend(
            [
                secret_key_validator.EmptyStringValidator(),
                secret_key_validator.WrongLengthValidator(),
            ]
        )


class ContentValidatorHandler(ValidatorHandler):
    def __init__(self):
        super().__init__()
        self.validators.extend(
            [
                content_validator.EmptyStringValidator(),
            ]
        )



import json
import os
import urllib
import urllib.error

import pytest
from translation_hub.apis.deepl_api import DeeplApi
from translation_hub.core.enums import Languages
from unittest.mock import patch
from translation_hub import exceptions
from translation_hub.config import cfg


@pytest.fixture
def deepl_api():
    return DeeplApi(api_key=cfg.get(cfg.DeeplApiKey))


class TestDeeplApi:
    @patch.dict(os.environ, {"DeeplApiKey": "env_api_key"})
    def test_get_api_key_from_env(self):
        api = DeeplApi()
        assert api.api_key == "env_api_key"

    @patch.object(
        cfg,
        "get",
        side_effect=lambda key: "cfg_api_key" if key == cfg.DeeplApiKey else None,
    )
    def test_get_api_key_from_cfg(self, mock_cfg):
        api = DeeplApi()
        assert api.api_key == "cfg_api_key"

    def test_get_api_key_directly(self):
        api = DeeplApi(api_key="direct_api_key")
        assert api.api_key == "direct_api_key"

    @patch.dict(os.environ, {}, clear=True)
    @patch.object(cfg, "get", return_value=None)
    def test_get_api_key_error(self, mock_cfg):
        with pytest.raises(exceptions.InvalidSecretKeyError):
            DeeplApi()

    def test_translates_text_correctly(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"translations": [{"text": "你好，世界"}]}
            )
            result = deepl_api.translate(
                "Hello, world!", Languages.English, Languages.Chinese
            )
            assert result == "你好，世界"

    def test_translates_text_correctly_with_network(self, deepl_api):
        result = deepl_api.translate(
            "Hello, world!", Languages.English, Languages.Chinese
        )
        assert result == "你好，世界"

    def test_translates_text_correctly_with_auto_language(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {
                    "translations": [
                        {"detected_source_language": "ZH", "text": "Hello, world."}
                    ]
                }
            )
            result = deepl_api.translate(
                "Hello, world!", Languages.Auto, Languages.English
            )
            assert result == "Hello, world."

    def test_handles_key_error(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.return_value = "{}"
            with pytest.raises(exceptions.ResponseError):
                deepl_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_json_decode_error(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.return_value = "Invalid JSON"
            with pytest.raises(exceptions.JsonDecodeError):
                deepl_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_generic_exception(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.side_effect = Exception("Some error")
            with pytest.raises(exceptions.UnknownError):
                deepl_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_translates_with_string_language(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"translations": [{"text": "你好，世界"}]}
            )
            result = deepl_api.translate("Hello, world!", "en", "zh")
            assert result == "你好，世界"

    def test_handles_request_error_403(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.side_effect = urllib.error.HTTPError(
                url="", code=403, msg="", hdrs={}, fp=None
            )
            with pytest.raises(exceptions.RequestError):
                deepl_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_request_error_429(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.side_effect = urllib.error.HTTPError(
                url="", code=429, msg="", hdrs={}, fp=None
            )
            with pytest.raises(exceptions.RequestError):
                deepl_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_request_error_456(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.side_effect = urllib.error.HTTPError(
                url="", code=456, msg="", hdrs={}, fp=None
            )
            with pytest.raises(exceptions.RequestError):
                deepl_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_request_error_500(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.side_effect = urllib.error.HTTPError(
                url="", code=500, msg="", hdrs={}, fp=None
            )
            with pytest.raises(exceptions.ServerError):
                deepl_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )

    def test_handles_unknown_error(self, deepl_api):
        with patch("translation_hub.apis.deepl_api.request") as mock_request:
            mock_request.side_effect = urllib.error.HTTPError(
                url="", code=999, msg="", hdrs={}, fp=None
            )
            with pytest.raises(exceptions.UnknownError):
                deepl_api.translate(
                    "Hello, world!", Languages.English, Languages.Chinese
                )



from abc import ABC, abstractmethod
from translation_hub.core.enums import Languages


class Api(ABC):
    api_url: str = ""
    api_key: str = ""

    @abstractmethod
    def translate(
        self,
        text: str,
        source: Languages | str = Languages.English,
        target: Languages | str = Languages.Chinese,
    ) -> str:
        pass



from translation_hub.api import Api
from translation_hub.core.enums import Languages
from translation_hub.utils.requests import request
from translation_hub import exceptions
from hashlib import md5
import random
import json
import logging
from translation_hub.validator_handler import (
    SecretKeyValidatorHandler,
    ContentValidatorHandler,
)
import os
from translation_hub.config import cfg


class BaiduAPI(Api):
    """
    百度翻译 API

    产品说明:
        免费额度: 200万 字/月
        开通地址: https://api.fanyi.baidu.com/product/11
        文档地址: https://api.fanyi.baidu.com/doc/21

    使用说明:
        百度翻译 API 通过百度翻译开放平台提供，需要注册账号并创建应用获取 API ID 和 API Key。

    正确返回值示例:
        {'from': 'en', 'to': 'zh', 'trans_result': [{'src': 'Hello', 'dst': '你好'}]}

    错误返回值示例:
        {"error_code":"54001","error_msg":"Invalid Sign"}
    """

    api_url: str = "http://api.fanyi.baidu.com/api/trans/vip/translate"

    def __init__(self, api_id: str | None = None, api_key: str | None = None):
        self.api_id, self.api_key = self._get_api_id_and_key(api_id, api_key)

        # 验证 API ID 和 API Key
        self.content_validator_handler = ContentValidatorHandler()
        self.secret_key_validator_handler = SecretKeyValidatorHandler()
        is_api_id_valid: bool = self.secret_key_validator_handler.validate(self.api_id)
        is_api_key_valid: bool = self.secret_key_validator_handler.validate(
            self.api_key
        )

        if not is_api_id_valid or not is_api_key_valid:
            raise exceptions.InvalidSecretKeyError(
                f"Invalid API ID or API Key, please check and try again."
                f"current api_id: {self.api_id}, current api_key: {self.api_key}"
            )

    def translate(
        self,
        text: str,
        source: Languages | str = Languages.English,
        target: Languages | str = Languages.Chinese,
    ) -> str:
        text = text.strip()
        is_content_valid: bool = self.content_validator_handler.validate(text)
        if not is_content_valid:
            raise exceptions.InvalidContentError(f"Invalid content: {text}")

        if isinstance(source, Languages):
            source = source.value
        if isinstance(target, Languages):
            target = target.value

        salt = random.randint(32768, 65536)

        sign = md5(f"{self.api_id}{text}{salt}{self.api_key}".encode()).hexdigest()

        payload = {
            "appid": self.api_id,
            "q": text,
            "from": source,
            "to": target,
            "salt": salt,
            "sign": sign,
        }

        headers = {"Content-Type": "application/x-www-form-urlencoded"}

        try:
            response = request(self.api_url, headers, payload=payload)
            response_data = json.loads(response)
            logging.debug(f"请求成功: {response_data}")
        except json.JSONDecodeError as e:
            raise exceptions.JsonDecodeError(f"Failed to decode response: {e}")
        except Exception as e:
            raise exceptions.RequestError(str(e))

        if "error_code" in response_data:
            error_code = response_data["error_code"]
            error_msg = response_data.get("error_msg", "Unknown error")

            match error_code:
                case "52001":
                    raise exceptions.ServerError(
                        "请求超时: 检查请求query是否超长，以及原文或译文参数是否在支持的语种列表里"
                    )
                case "52002":
                    raise exceptions.ServerError("服务器系统错误: 请重试")
                case "52003":
                    raise exceptions.ServerError(
                        "未授权用户: 请检查appid是否正确或者服务是否开通"
                    )
                case "54000":
                    raise exceptions.ServerError("必填参数为空: 请检查是否少传参数")
                case "54001":
                    raise exceptions.ServerError("签名错误: 请检查您的签名生成方法")
                case "54003":
                    raise exceptions.ServerError(
                        "访问频率受限: 请降低您的调用频率，或在控制台进行身份认证后切换为高级版/尊享版"
                    )
                case "54004":
                    raise exceptions.ServerError(
                        "账户余额不足: 请前往管理控制台为账户充值"
                    )
                case "54005":
                    raise exceptions.ServerError(
                        "长query请求频繁: 请降低长query的发送频率，3s后再试"
                    )
                case "58000":
                    raise exceptions.ServerError(
                        "客户端IP非法: 检查个人资料里填写的IP地址是否正确，可前往开发者信息-基本信息修改"
                    )
                case "58001":
                    raise exceptions.ServerError(
                        "译文语言方向不支持: 检查译文语言是否在语言列表里"
                    )
                case "58002":
                    raise exceptions.ServerError(
                        "服务当前已关闭: 请前往管理控制台开启服务"
                    )
                case "58003":
                    raise exceptions.ServerError(
                        "此IP已被封禁: 同一IP当日使用多个APPID发送翻译请求，则该IP将被封禁当日请求权限，次日解封。请勿将APPID和密钥填写到第三方软件中。"
                    )
                case "90107":
                    raise exceptions.ServerError(
                        "认证未通过或未生效: 请前往我的认证查看认证进度"
                    )
                case "20003":
                    raise exceptions.ServerError("请求内容存在安全风险: 请检查请求内容")
                case _:
                    raise exceptions.UnknownError(
                        f"Error code: {error_code}, message: {error_msg}"
                    )

        try:
            result = response_data["trans_result"][0]["dst"]
        except KeyError:
            raise exceptions.ResponseError(f"Failed to get translation: {response}")
        except Exception as e:
            raise exceptions.UnknownError(str(e))
        return result

    def _get_api_id_and_key(
        self, api_id: str | None, api_key: str | None
    ) -> tuple[str, str]:
        """先从环境变量中获取，如果没有则从配置文件中获取"""
        if api_id and api_key:
            return api_id, api_key

        api_id = os.environ.get("BaiduAppId")
        api_key = os.environ.get("BaiduSecretKey")
        if api_id and api_key:
            return api_id, api_key

        api_id = cfg.get(cfg.BaiduAppId)
        api_key = cfg.get(cfg.BaiduSecretKey)
        if api_id and api_key:
            return api_id, api_key

        raise exceptions.InvalidSecretKeyError(
            f"Invalid API ID or API Key, please check and try again."
            f"current api_id: {api_id}, current api_key: {api_key}"
        )



import json
import os
import pytest
from translation_hub.apis.baidu_api import BaiduAPI
from translation_hub.core.enums import Languages
from unittest.mock import patch
from translation_hub import exceptions
from translation_hub.config import cfg


@pytest.fixture
def baidu_api():
    return BaiduAPI(api_id=cfg.get(cfg.BaiduAppId), api_key=cfg.get(cfg.BaiduSecretKey))


class TestBaiduAPI:
    @patch.dict(
        os.environ, {"BaiduAppId": "env_api_id", "BaiduSecretKey": "env_api_key"}
    )
    def test_get_api_id_and_key_from_env(self):
        api = BaiduAPI()
        assert api.api_id == "env_api_id"
        assert api.api_key == "env_api_key"

    @patch.object(
        cfg,
        "get",
        side_effect=lambda key: "cfg_api_id"
        if key == cfg.BaiduAppId
        else "cfg_api_key",
    )
    def test_get_api_id_and_key_from_cfg(self, mock_cfg):
        api = BaiduAPI()
        assert api.api_id == "cfg_api_id"
        assert api.api_key == "cfg_api_key"

    def test_get_api_id_and_key_directly(self):
        api = BaiduAPI(api_id="direct_api_id", api_key="direct_api_key")
        assert api.api_id == "direct_api_id"
        assert api.api_key == "direct_api_key"

    @patch.dict(os.environ, {}, clear=True)
    @patch.object(cfg, "get", return_value=None)
    def test_get_api_id_and_key_error(self, mock_cfg):
        with pytest.raises(exceptions.InvalidSecretKeyError):
            BaiduAPI()

    def test_translates_text_correctly(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {
                    "from": "en",
                    "to": "zh",
                    "trans_result": [{"src": "Hello", "dst": "你好"}],
                }
            )
            result = baidu_api.translate("Hello", Languages.English, Languages.Chinese)
            assert result == "你好"

    def test_translates_text_with_network(self, baidu_api):
        result = baidu_api.translate(
            "这是一个测试", Languages.Chinese, Languages.English
        )
        assert result == "This is a test"

    def test_translates_blank_text(self, baidu_api):
        with pytest.raises(exceptions.InvalidContentError):
            baidu_api.translate("", Languages.English, Languages.Chinese)

    def test_handles_key_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = "{}"
            with pytest.raises(exceptions.ResponseError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_json_decode_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = "Invalid JSON"
            with pytest.raises(exceptions.JsonDecodeError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_generic_exception(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.side_effect = Exception("Some error")
            with pytest.raises(exceptions.RequestError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_translates_with_string_language(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps({"trans_result": [{"dst": "你好"}]})
            result = baidu_api.translate("Hello", "en", "zh")
            assert result == "你好"

    def test_handles_request_timeout_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "52001", "error_msg": "请求超时"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_system_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "52002", "error_msg": "系统错误"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_unauthorized_user_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "52003", "error_msg": "未授权用户"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_missing_parameter_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "54000", "error_msg": "必填参数为空"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_signature_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "54001", "error_msg": "签名错误"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_rate_limit_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "54003", "error_msg": "访问频率受限"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_insufficient_balance_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "54004", "error_msg": "账户余额不足"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_frequent_long_query_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "54005", "error_msg": "长query请求频繁"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_invalid_client_ip_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "58000", "error_msg": "客户端IP非法"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_unsupported_language_direction_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "58001", "error_msg": "译文语言方向不支持"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_service_closed_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "58002", "error_msg": "服务当前已关闭"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_ip_banned_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "58003", "error_msg": "此IP已被封禁"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_authentication_failed_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "90107", "error_msg": "认证未通过或未生效"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_security_risk_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "20003", "error_msg": "请求内容存在安全风险"}
            )
            with pytest.raises(exceptions.ServerError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)

    def test_handles_unknown_error(self, baidu_api):
        with patch("translation_hub.apis.baidu_api.request") as mock_request:
            mock_request.return_value = json.dumps(
                {"error_code": "99999", "error_msg": "未知错误"}
            )
            with pytest.raises(exceptions.UnknownError):
                baidu_api.translate("Hello", Languages.English, Languages.Chinese)



# coding:utf-8
import contextlib
import json
import sys
from copy import deepcopy
from enum import Enum
from pathlib import Path
from typing import List
from translation_hub.core.paths import CONFIG_FILE


def exception_handler(*default):
    """decorator for exception handling

    Parameters
    ----------
    *default:
        the default value returned when an exception occurs
    """

    def outer(func):
        def inner(*args, **kwargs):
            with contextlib.suppress(BaseException):
                return func(*args, **kwargs)
            value = deepcopy(default)
            if len(value) == 0:
                return None
            elif len(value) == 1:
                return value[0]
            return value

        return inner

    return outer


class ConfigValidator:
    """Config validator"""

    def validate(self, value):
        """Verify whether the value is legal"""
        return True

    def correct(self, value):
        """correct illegal value"""
        return value


class RangeValidator(ConfigValidator):
    """Range validator"""

    def __init__(self, min_value, max_value):
        self.min = min_value
        self.max = max_value
        self.range = (min_value, max_value)

    def validate(self, value):
        return self.min <= value <= self.max

    def correct(self, value):
        return min(max(self.min, value), self.max)


class OptionsValidator(ConfigValidator):
    """Options validator"""

    def __init__(self, options):
        if not options:
            raise ValueError("The `options` can't be empty.")

        if isinstance(options, Enum):
            options = options._member_map_.values()

        self.options = list(options)

    def validate(self, value):
        return value in self.options

    def correct(self, value):
        return value if self.validate(value) else self.options[0]


class BoolValidator(OptionsValidator):
    """Boolean validator"""

    def __init__(self):
        super().__init__([True, False])


class FolderValidator(ConfigValidator):
    """Folder validator"""

    def validate(self, value):
        return Path(value).exists()

    def correct(self, value):
        path = Path(value)
        path.mkdir(exist_ok=True, parents=True)
        return str(path.absolute()).replace("\\", "/")


class FolderListValidator(ConfigValidator):
    """Folder list validator"""

    def validate(self, value):
        return all(Path(i).exists() for i in value)

    def correct(self, value: List[str]):
        folders = []
        for folder in value:
            path = Path(folder)
            if path.exists():
                folders.append(str(path.absolute()).replace("\\", "/"))

        return folders


class ConfigSerializer:
    """Config serializer"""

    def serialize(self, value):
        """serialize config value"""
        return value

    def deserialize(self, value):
        """deserialize config from config file's value"""
        return value


class EnumSerializer(ConfigSerializer):
    """enumeration class serializer"""

    def __init__(self, enum_class):
        self.enumClass = enum_class

    def serialize(self, value):
        return value.value

    def deserialize(self, value):
        return self.enumClass(value)


class ConfigItem:
    """Config item"""

    def __init__(
        self, group, name, default, validator=None, serializer=None, restart=False
    ):
        """
        Parameters
        ----------
        group: str
            config group name

        name: str
            config item name, can be empty

        default:
            default value

        options: list
            options value

        serializer: ConfigSerializer
            config serializer

        restart: bool
            whether to restart the application after updating value
        """
        super().__init__()
        self.group = group
        self.name = name
        self.validator = validator or ConfigValidator()
        self.serializer = serializer or ConfigSerializer()
        self.__value = default
        self.value = default
        self.restart = restart
        self.defaultValue = self.validator.correct(default)

    @property
    def value(self):
        """get the value of config item"""
        return self.__value

    @value.setter
    def value(self, v):
        v = self.validator.correct(v)
        self.__value = v

    @property
    def key(self):
        """get the config key separated by `.`"""
        return self.group + "." + self.name if self.name else self.group

    def __str__(self):
        return f"{self.__class__.__name__}[value={self.value}]"

    def serialize(self):
        return self.serializer.serialize(self.value)

    def deserialize_from(self, value):
        self.value = self.serializer.deserialize(value)


class RangeConfigItem(ConfigItem):
    """Config item of range"""

    validator: RangeValidator

    @property
    def range(self):
        """get the available range of config"""
        return self.validator.range

    def __str__(self):
        return f"{self.__class__.__name__}[range={self.range}, value={self.value}]"


class OptionsConfigItem(ConfigItem):
    """Config item with options"""

    validator: OptionsValidator

    @property
    def options(self):
        return self.validator.options

    def __str__(self):
        return f"{self.__class__.__name__}[options={self.options}, value={self.value}]"


class QConfig:
    """Config of app"""

    def __init__(self):
        super().__init__()
        self.file = CONFIG_FILE
        self._cfg = self

    @staticmethod
    def get(item):
        """get the value of config item"""
        return item.value

    def set(self, item, value, save=True, copy=True):
        """set the value of config item

        Parameters
        ----------
        item: ConfigItem
            config item

        value:
            the new value of config item

        save: bool
            whether to save the change to config file

        copy: bool
            whether to deep copy the new value
        """
        if item.value == value:
            return

        # deepcopy new value
        try:
            item.value = deepcopy(value) if copy else value
        except Exception:
            item.value = value

        if save:
            self.save()

        if item.restart:
            sys.exit(0)

    def to_dict(self, serialize=True):
        """convert config items to `dict`"""
        items = {}
        for name in dir(self._cfg.__class__):
            item = getattr(self._cfg.__class__, name)
            if not isinstance(item, ConfigItem):
                continue

            value = item.serialize() if serialize else item.value
            if not items.get(item.group):
                if not item.name:
                    items[item.group] = value
                else:
                    items[item.group] = {}

            if item.name:
                items[item.group][item.name] = value

        return items

    def save(self):
        """save config"""
        self._cfg.file.parent.mkdir(parents=True, exist_ok=True)
        with open(self._cfg.file, "w", encoding="utf-8") as f:
            json.dump(self._cfg.to_dict(), f, ensure_ascii=False, indent=4)

    @exception_handler()
    def load(self, file=None, config=None):
        """load config

        Parameters
        ----------
        file: str or Path
            the path of json config file

        config: Config
            config object to be initialized
        """
        if isinstance(config, QConfig):
            self._cfg = config

        if isinstance(file, (str, Path)):
            self._cfg.file = Path(file)

        try:
            with open(self._cfg.file, encoding="utf-8") as f:
                cfg = json.load(f)
        except Exception:
            cfg = {}

        # map config items'key to item
        items = {}
        for name in dir(self._cfg.__class__):
            item = getattr(self._cfg.__class__, name)
            if isinstance(item, ConfigItem):
                items[item.key] = item

        # update the value of config item
        for k, v in cfg.items():
            if not isinstance(v, dict) and items.get(k) is not None:
                items[k].deserialize_from(v)
            elif isinstance(v, dict):
                for key, value in v.items():
                    key = k + "." + key
                    if items.get(key) is not None:
                        items[key].deserialize_from(value)


qconfig = QConfig()



import hashlib
import hmac
import json
import os
import time

from translation_hub import exceptions
from translation_hub.api import Api
from translation_hub.config import cfg
from translation_hub.core.enums import Languages
from translation_hub.validator_handler import (
    SecretKeyValidatorHandler,
    ContentValidatorHandler,
)
import urllib
import urllib.error
import urllib.request
import logging


class TencentApi(Api):
    """
    腾讯翻译 API

    产品说明:
        免费额度: 500万 字/月
        开通地址: https://console.cloud.tencent.com/cam/capi
        文档地址: https://cloud.tencent.com/document/product/551

    使用说明:
        腾讯财大气粗,每月免费额度高达 500 万字,appid 和 secretkey 需要在腾讯云控制台申请
        教程: https://cloud.tencent.com.cn/developer/information/腾讯翻译api申请教程-article

    正确返回值示例:
        {
            "Response": {
                "RequestId": "5a94cad9-48e3-4318-b57e-2f7c59c83929",
                "Source": "zh",
                "Target": "en",
                "TargetText": "Hello world!",
                "UsedAmount": 6
            }
        }

    错误返回值示例:
        {
            "Response": {
                "Error": {
                    "Code": "AuthFailure.SecretIdNotFound",
                    "Message": "The SecretId is not found, please ensure that your SecretId is correct."
                },
                "RequestId": "f93b6a33-9a12-46cc-bb19-46c086a21ec6"
            }
        }
    """

    api_url = "https://tmt.tencentcloudapi.com/"

    def __init__(self, api_id: str | None = None, api_key: str | None = None):
        self.api_id, self.api_key = self._get_api_id_and_key(api_id, api_key)

        self.content_validator_handler = ContentValidatorHandler()
        secret_key_validator_handler = SecretKeyValidatorHandler()
        # 验证 API ID 和 API Key
        is_api_id_valid = secret_key_validator_handler.validate(self.api_id)
        is_api_key_valid = secret_key_validator_handler.validate(self.api_key)

        if not is_api_id_valid or not is_api_key_valid:
            raise exceptions.InvalidSecretKeyError(
                f"Invalid API ID or API Key, please check and try again."
                f"current api_id: {self.api_id}, current api_key: {self.api_key}"
            )

    def translate(
        self,
        text: str,
        source: Languages | str = Languages.English,
        target: Languages | str = Languages.Chinese,
    ) -> str:
        """
        调用腾讯翻译 API 进行翻译。

        参数:
            text (str): 待翻译的文本。
            source (Languages, optional): 源语言，默认为英语。
            target (Languages, optional): 目标语言，默认为中文。

        返回:
            str: 翻译后的文本。
        """
        # 将语言转换为腾讯翻译 API 支持的语言代码
        text = text.strip()

        is_content_valid: bool = self.content_validator_handler.validate(text)
        if not is_content_valid:
            raise exceptions.InvalidContentError(f"Invalid content: {text}")

        source = self._trans_language(source)
        target = self._trans_language(target)

        # 腾讯的API非常麻烦，看得出非常想让别人去用它那个庞大的 SDK

        # 构造请求参数
        payload = {
            "SourceText": text,
            "Source": source,
            "Target": target,
            "ProjectId": 0,
        }

        # 构造请求头
        timestamp = int(time.time())
        headers = {
            "Authorization": self._get_authorization(payload, timestamp),
            "Content-Type": "application/json; charset=utf-8",
            "Host": "tmt.tencentcloudapi.com",
            "X-TC-Action": "TextTranslate",
            "X-TC-Version": "2018-03-21",
            "X-TC-Timestamp": str(timestamp),
            "X-TC-Region": "ap-guangzhou",
        }

        try:
            request_data = json.dumps(payload).encode("utf-8")
            req = urllib.request.Request(
                self.api_url, data=request_data, headers=headers, method="POST"
            )
            with urllib.request.urlopen(req) as response:
                response = response.read().decode("utf-8")
            logging.debug(f"请求成功: {response}")
        except urllib.error.HTTPError as e:
            raise exceptions.RequestError(str(e))
        except Exception as e:
            raise exceptions.UnknownError(str(e))

        try:
            response_data = json.loads(response)
            if "Error" in response_data["Response"]:
                raise exceptions.ServerError(
                    f"服务出现错误: {response_data['Response']['Error']}"
                )
            return response_data["Response"]["TargetText"]
        except json.JSONDecodeError as e:
            raise exceptions.JsonDecodeError(f"Failed to decode response: {e}")
        except KeyError as e:
            raise exceptions.ResponseError(f"Response error: {response}, error: {e}")

    def _trans_language(self, languages: Languages | str) -> str:
        """转换语言代码为支持的格式"""
        if isinstance(languages, Languages):
            languages = languages.value

        trans_dict: dict[str, str] = {
            "auto": "auto",
            "zh": "zh",
            "en": "en",
            "ru": "ru",
            "ja": "ja",
            "kor": "ko",
        }

        return trans_dict.get(languages) or languages

    def _get_api_id_and_key(
        self, api_id: str | None = None, api_key: str | None = None
    ) -> tuple[str, str]:
        """先从环境变量中获取，如果没有则从配置文件中获取"""
        if api_id and api_key:
            return api_id, api_key

        api_id = os.environ.get("TencentAppId")
        api_key = os.environ.get("TencentSecretKey")
        if api_id and api_key:
            return api_id, api_key

        api_id = cfg.get(cfg.TencentAppId)
        api_key = cfg.get(cfg.TencentSecretKey)
        if api_id and api_key:
            return api_id, api_key

        raise exceptions.InvalidSecretKeyError(
            f"Invalid API ID or API Key, please check and try again."
            f"current api_id: {api_id}, current api_key: {api_key}"
        )

    def _get_authorization(self, payload: dict, timestamp: int) -> str:
        """获取请求头中的 Authorization 字段"""
        # 公共参数
        host = "tmt.tencentcloudapi.com"
        service = "tmt"
        algorithm = "TC3-HMAC-SHA256"
        date = time.strftime("%Y-%m-%d", time.gmtime(timestamp))

        # ************* 步骤 1：拼接规范请求串 *************
        http_request_method = "POST"
        canonical_uri = "/"
        canonical_querystring = ""
        canonical_headers = (
            "content-type:application/json; charset=utf-8\nhost:" + host + "\n"
        )
        signed_headers = "content-type;host"
        hashed_request_payload = hashlib.sha256(
            json.dumps(payload).encode("utf-8")
        ).hexdigest()
        canonical_request = (
            http_request_method
            + "\n"
            + canonical_uri
            + "\n"
            + canonical_querystring
            + "\n"
            + canonical_headers
            + "\n"
            + signed_headers
            + "\n"
            + hashed_request_payload
        )

        # ************* 步骤 2：拼接待签名字符串 *************
        credential_scope = date + "/" + service + "/tc3_request"
        hashed_canonical_request = hashlib.sha256(
            canonical_request.encode("utf-8")
        ).hexdigest()
        string_to_sign = (
            algorithm
            + "\n"
            + str(timestamp)
            + "\n"
            + credential_scope
            + "\n"
            + hashed_canonical_request
        )

        # ************* 步骤 3：计算签名 *************
        def sign(key, msg):
            return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()

        secret_date = sign(("TC3" + self.api_key).encode("utf-8"), date)
        secret_service = sign(secret_date, service)
        secret_signing = sign(secret_service, "tc3_request")
        signature = hmac.new(
            secret_signing, string_to_sign.encode("utf-8"), hashlib.sha256
        ).hexdigest()

        # ************* 步骤 4：构造 Authorization 头部 *************
        authorization = (
            algorithm
            + " "
            + "Credential="
            + self.api_id
            + "/"
            + credential_scope
            + ", "
            + "SignedHeaders="
            + signed_headers
            + ", "
            + "Signature="
            + signature
        )

        return authorization



from translation_hub.validators.validator import Validator


class EmptyStringValidator(Validator):
    """空字符串验证器"""

    def validate(self, value: str) -> bool:
        return bool(value.strip())


class WrongLengthValidator(Validator):
    """长度验证器"""

    def validate(self, value: str, min_length: int = 10, max_length: int = 100) -> bool:
        return min_length <= len(value) <= max_length


class NumberOnlyValidator(Validator):
    """纯数字验证器"""

    def validate(self, value: str) -> bool:
        return value.isdigit()


class LetterOnlyValidator(Validator):
    """纯字母验证器"""

    def validate(self, value: str) -> bool:
        return value.isalpha()


class HasNumberAndLetterValidator(Validator):
    """同时包含数字和字母验证器"""

    def validate(self, value: str) -> bool:
        return any(c.isdigit() for c in value) and any(c.isalpha() for c in value)


class HasSpecialCharValidator(Validator):
    """特殊字符验证器"""

    def validate(self, value: str) -> bool:
        return not value.isalnum()



import logging
from translation_hub.apis.google_api import GoogleApi
from translation_hub.apis.aliyun_api import AliyunApi
from translation_hub.apis.baidu_api import BaiduAPI
from translation_hub.apis.tencent_api import TencentApi
from translation_hub.apis.youdao_api import YoudaoApi
from translation_hub.apis.deepl_api import DeeplApi
from translation_hub.core.enums import Languages

__all__ = [
    "GoogleApi",
    "AliyunApi",
    "BaiduAPI",
    "TencentApi",
    "YoudaoApi",
    "DeeplApi",
    Languages,
]

logging.basicConfig(
    level=logging.DEBUG, format="%(asctime)s - %(levelname)s - %(message)s"
)



import json
import os
import pytest
from translation_hub.apis.youdao_api import YoudaoApi
from translation_hub.core.enums import Languages
from unittest.mock import patch
from translation_hub import exceptions
from translation_hub.config import cfg


@pytest.fixture
def youdao_api():
    return YoudaoApi(
        api_id=cfg.get(cfg.YoudaoAppId), api_key=cfg.get(cfg.YoudaoSecretKey)
    )


class TestYoudaoApi:
    @patch.dict(
        os.environ, {"YoudaoAppId": "env_api_id", "YoudaoSecretKey": "env_api_key"}
    )
    def test_get_api_id_and_key_from_env(self):
        api = YoudaoApi()
        assert api.api_id == "env_api_id"
        assert api.api_key == "env_api_key"

    @patch.object(
        cfg,
        "get",
        side_effect=lambda key: "cfg_api_id"
        if key == cfg.YoudaoAppId
        else "cfg_api_key",
    )
    def test_get_api_id_and_key_from_cfg(self, mock_cfg):
        api = YoudaoApi()
        assert api.api_id == "cfg_api_id"
        assert api.api_key == "cfg_api_key"

    def test_get_api_id_and_key_directly(self):
        api = YoudaoApi(api_id="direct_api_id", api_key="direct_api_key")
        assert api.api_id == "direct_api_id"
        assert api.api_key == "direct_api_key"

    @patch.dict(os.environ, {}, clear=True)
    @patch.object(cfg, "get", return_value=None)
    def test_get_api_id_and_key_error(self, mock_cfg):
        with pytest.raises(exceptions.InvalidSecretKeyError):
            YoudaoApi()

    def test_translates_text_correctly(self, youdao_api):
        result = youdao_api.translate("你好,世界", Languages.Chinese, Languages.English)
        assert result == "Hello, world"

    def test_handles_key_error(self, youdao_api):
        with patch("translation_hub.apis.youdao_api.request") as mock_request:
            mock_request.return_value = "{}"
            with pytest.raises(exceptions.JsonDecodeError):
                youdao_api.translate("你好,世界", Languages.Chinese, Languages.English)

    def test_handles_json_decode_error(self, youdao_api):
        with patch("translation_hub.apis.youdao_api.request") as mock_request:
            mock_request.return_value = "Invalid JSON"
            with pytest.raises(exceptions.JsonDecodeError):
                youdao_api.translate("你好,世界", Languages.Chinese, Languages.English)

    def test_handles_generic_exception(self, youdao_api):
        with patch("translation_hub.apis.youdao_api.request") as mock_request:
            mock_request.side_effect = Exception("Some error")
            with pytest.raises(exceptions.UnknownError):
                youdao_api.translate("你好,世界", Languages.Chinese, Languages.English)

    def test_handles_request_error_101(self, youdao_api):
        with patch("translation_hub.apis.youdao_api.request") as mock_request:
            mock_request.return_value = json.dumps({"errorCode": "101"})
            with pytest.raises(exceptions.RequestError):
                youdao_api.translate("你好,世界", Languages.Chinese, Languages.English)

    def test_handles_request_error_102(self, youdao_api):
        with patch("translation_hub.apis.youdao_api.request") as mock_request:
            mock_request.return_value = json.dumps({"errorCode": "102"})
            with pytest.raises(exceptions.RequestError):
                youdao_api.translate("你好,世界", Languages.Chinese, Languages.English)

    def test_handles_request_error_103(self, youdao_api):
        with patch("translation_hub.apis.youdao_api.request") as mock_request:
            mock_request.return_value = json.dumps({"errorCode": "103"})
            with pytest.raises(exceptions.RequestError):
                youdao_api.translate("你好,世界", Languages.Chinese, Languages.English)

    def test_handles_unknown_error(self, youdao_api):
        with patch("translation_hub.apis.youdao_api.request") as mock_request:
            mock_request.return_value = json.dumps({"errorCode": "99999"})
            with pytest.raises(exceptions.RequestError):
                youdao_api.translate("你好,世界", Languages.Chinese, Languages.English)
